SystemOrganization addCategory: #'AL-Storage-Core'!
SystemOrganization addCategory: #'AL-Storage-Exceptions'!
SystemOrganization addCategory: #'AL-Storage-Repository'!
SystemOrganization addCategory: #'AL-Storage-Utilities'!

ALRelay subclass: #ALStorageRelay
	instanceVariableNames: 'repository txn'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AL-Storage-Core'!

!ALStorageRelay class methodsFor: 'as yet unclassified' stamp: 'cwp 11/25/2012 09:53'!
destination: aRelay repository: aRepository
	^ self basicNew initializeWithDestination: aRelay repository: aRepository! !

!ALStorageRelay methodsFor: 'relaying' stamp: 'cwp 1/12/2013 00:15'!
etagForResponse: response
	| dependencies digest fields out |
	txn := repository readOnlyTransaction.
	txn do: [ response basicWriteBodyInto: nil writing ].
	out := Array new writing.
	txn reads keysAndValuesDo:
		[:key :vector |
		out put: key.
		out put: vector version].
	dependencies := out conclusion.
	dependencies ifEmpty: [ ^ nil ].
	digest := dependencies serialize digest printAsKey.
	fields := response vary.
	fields ifNotNil: 
		[(fields includes: 'Accept') ifTrue: 
			[digest := digest , ',' , response contentType printString]].
	^ ALEtag weak: digest! !

!ALStorageRelay methodsFor: 'relaying' stamp: 'cwp 1/12/2013 19:38'!
handleRequest: aRequest
	| response etag match |
	response := repository transaction commit: 
		[destination handleRequest: aRequest].
	(self shouldTag: response) ifTrue: 
		[etag := self etagForResponse: response.
		etag ifNotNil: 
			[match := aRequest ifNoneMatch.
			(match notNil and: [match tag = etag tag]) ifTrue: 
				[response := self notModified: response].
			response etag: etag]].
	^ response! !

!ALStorageRelay methodsFor: 'initialization' stamp: 'cwp 11/25/2012 09:54'!
initializeWithDestination: aRelay repository: aRepository
	self initializeWithDestination: aRelay.
	repository := aRepository! !

!ALStorageRelay methodsFor: 'relaying' stamp: 'cwp 1/12/2013 19:38'!
notModified: aResponse
	| replacement |
	replacement := ALResponse status: 304 reason: 'Not modified'.
	aResponse vary ifNotNil: [ :v | replacement vary: v ].
	^ replacement! !

!ALStorageRelay methodsFor: 'relaying' stamp: 'cwp 12/19/2012 22:38'!
shouldTag: response
	^ response hasEntity 
		and: [response hasLastModified not 
			and: [response hasEtag not]]! !

!ALStorageRelay methodsFor: 'relaying' stamp: 'cwp 1/2/2013 23:08'!
write: aMessage into: aWriteStream
	txn ifNil: [txn := repository readOnlyTransaction].
	txn do: [ destination write: aMessage into: aWriteStream ]! !

Notification subclass: #ALStorageNotification
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AL-Storage-Exceptions'!

ALStorageNotification subclass: #ALAddEntity
	instanceVariableNames: 'vector'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AL-Storage-Exceptions'!

!ALAddEntity class methodsFor: 'signaling' stamp: 'cwp 12/31/2012 10:14'!
vector: aVector
	^ (self withVector: aVector) signal! !

!ALAddEntity class methodsFor: 'instance creation' stamp: 'cwp 12/31/2012 10:14'!
withVector: aVector
	^ self basicNew initializeWithVector: aVector! !

!ALAddEntity methodsFor: 'as yet unclassified' stamp: 'cwp 11/24/2012 23:09'!
handleWith: anObject
	^ anObject handleAddEntity: self! !

!ALAddEntity methodsFor: 'as yet unclassified' stamp: 'cwp 12/31/2012 10:14'!
initializeWithVector: aVector
	self initialize.
	vector := aVector! !

!ALAddEntity methodsFor: 'as yet unclassified' stamp: 'cwp 12/31/2012 10:15'!
vector
	^ vector! !

ALStorageNotification subclass: #ALCollectEntities
	instanceVariableNames: 'keys'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AL-Storage-Exceptions'!

!ALCollectEntities class methodsFor: 'as yet unclassified' stamp: 'cwp 11/24/2012 23:27'!
keys: aCollection
	^ (self withKeys: aCollection) signal! !

!ALCollectEntities class methodsFor: 'as yet unclassified' stamp: 'cwp 11/24/2012 23:28'!
withKeys: aCollection
	^ self basicNew initializeWithKeys: aCollection! !

!ALCollectEntities methodsFor: 'as yet unclassified' stamp: 'cwp 11/24/2012 23:16'!
handleWith: anObject
	^ anObject handleCollectEntities: self! !

!ALCollectEntities methodsFor: 'as yet unclassified' stamp: 'cwp 11/24/2012 23:28'!
initializeWithKeys: aCollection
	self initialize.
	keys := aCollection! !

!ALCollectEntities methodsFor: 'as yet unclassified' stamp: 'cwp 11/24/2012 23:27'!
keys
	^ keys! !

ALStorageNotification subclass: #ALDeleteEntity
	instanceVariableNames: 'key'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AL-Storage-Exceptions'!

!ALDeleteEntity class methodsFor: 'as yet unclassified' stamp: 'cwp 11/24/2012 23:29'!
key: aKey
	^ (self withKey: aKey) signal! !

!ALDeleteEntity class methodsFor: 'as yet unclassified' stamp: 'cwp 11/24/2012 23:29'!
withKey: aKey
	^ self basicNew initializeWithKey: aKey! !

!ALDeleteEntity methodsFor: 'as yet unclassified' stamp: 'cwp 11/24/2012 23:09'!
handleWith: anObject
	^ anObject handleDeleteEntity: self! !

!ALDeleteEntity methodsFor: 'as yet unclassified' stamp: 'cwp 11/24/2012 23:29'!
initializeWithKey: aKey
	self initialize.
	key := aKey! !

!ALDeleteEntity methodsFor: 'as yet unclassified' stamp: 'cwp 11/24/2012 23:28'!
key
	^ key! !

ALStorageNotification subclass: #ALReadEntity
	instanceVariableNames: 'key'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AL-Storage-Exceptions'!

!ALReadEntity class methodsFor: 'as yet unclassified' stamp: 'cwp 11/24/2012 23:32'!
key: aKey
	^ (self withKey: aKey) signal! !

!ALReadEntity class methodsFor: 'as yet unclassified' stamp: 'cwp 11/24/2012 23:32'!
withKey: aKey
	^ self basicNew initializeWithKey: aKey! !

!ALReadEntity methodsFor: 'as yet unclassified' stamp: 'cwp 11/24/2012 23:10'!
handleWith: anObject
	^ anObject handleReadEntity: self! !

!ALReadEntity methodsFor: 'as yet unclassified' stamp: 'cwp 11/24/2012 23:32'!
initializeWithKey: aKey
	self initialize.
	key := aKey! !

!ALReadEntity methodsFor: 'as yet unclassified' stamp: 'cwp 11/24/2012 23:31'!
key
	^ key! !

!ALReadEntity methodsFor: 'as yet unclassified' stamp: 'cwp 11/27/2012 16:21'!
keys
	^ {key}! !

Error subclass: #ALStorageError
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AL-Storage-Exceptions'!

ALStorageError subclass: #ALImmutableError
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AL-Storage-Exceptions'!

!ByteArray methodsFor: '*al-storage-printing' stamp: 'cwp 11/29/2012 16:53'!
printAsKey
	| out |
	out := (String new: self size // 3 * 4) writing.
	self printAsKeyOn: out.
	^ out conclusion
	! !

!ByteArray methodsFor: '*al-storage-printing' stamp: 'cwp 11/29/2012 16:49'!
printAsKeyOn: out
	| alphabet in |
	alphabet := ALUrlStrategy alphabet.
	in := self reading.
	1 to: self size // 3 do: [ :ignore | 
		| byte bits |
		byte := in get.
		bits := byte bitShift: -2.
		out put: (alphabet at: bits + 1).
		bits := (byte bitAnd: 2r00000011) bitShift: 4.
		byte := in get.
		bits := bits bitOr: (byte bitShift: -4).
		out put: (alphabet at: bits + 1).
		bits := (byte bitAnd: 2r00001111) bitShift: 2.
		byte := in get.
		bits := bits bitOr: (byte bitShift: -6).
		out put: (alphabet at: bits + 1).
		bits := byte bitAnd: 2r00111111.
		out put: (alphabet at: bits + 1) ].
	! !

Object subclass: #ALEntity
	instanceVariableNames: 'key vector'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AL-Storage-Core'!

!ALEntity class methodsFor: 'configuration' stamp: 'cwp 1/11/2013 23:15'!
variableNames
	^ #()! !

!ALEntity methodsFor: 'as yet unclassified' stamp: 'cwp 12/31/2012 10:20'!
key
	^ key! !

!ALEntity methodsFor: 'as yet unclassified' stamp: 'cwp 1/4/2013 22:26'!
serializeWith: aSerializer
	aSerializer serialize: self with:
		[aSerializer 
			encode: self;
			serialize: key;
			serialize: nil]! !

Object subclass: #ALEntityGenerator
	instanceVariableNames: 'class'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AL-Storage-Utilities'!

!ALEntityGenerator class methodsFor: 'as yet unclassified' stamp: 'cwp 12/31/2012 09:38'!
generate: aClass
	^ (self on: aClass) generate! !

!ALEntityGenerator class methodsFor: 'as yet unclassified' stamp: 'cwp 12/31/2012 09:32'!
on: aClass
	^ self basicNew initializeWithClass: aClass! !

!ALEntityGenerator methodsFor: 'as yet unclassified' stamp: 'cwp 12/31/2012 09:46'!
accessorCategory
	^ 'accessing'! !

!ALEntityGenerator methodsFor: 'as yet unclassified' stamp: 'cwp 12/31/2012 09:54'!
compile: source classified: aString
	| existing selector |
	selector := Parser new parseSelector: source.
	existing := class sourceCodeAt: selector ifAbsent: [nil].
	source = existing asString ifFalse:
		[class compile: source classified: aString]! !

!ALEntityGenerator methodsFor: 'as yet unclassified' stamp: 'cwp 12/31/2012 09:46'!
deleteAccessors: names
	| out selectors |
	out := Array new writing.
	names do:
		[:ea |
		out put: ea asSymbol.
		out put: (ea copyWith: $:) asSymbol].
	selectors := out conclusion.
	(class organization listAtCategoryNamed: self accessorCategory) do:
		[:ea | (selectors includes: ea) ifFalse: [class removeSelector: ea]]! !

!ALEntityGenerator methodsFor: 'as yet unclassified' stamp: 'cwp 1/2/2013 19:04'!
generate
	| names |
	names := class variableNames.
	self deleteAccessors: names.
	names doWithIndex:
		[:name :index |
		self generateGetter: name index: index.
		self generateSetter: name index: index].
	self generateInitialize: names.
	self generateConstructor: names! !

!ALEntityGenerator methodsFor: 'as yet unclassified' stamp: 'cwp 1/2/2013 19:25'!
generateConstructor: names
	| out |
	out := String new writing.
	names do: 
		[:name | 
		out space.
		self writeSet: name on: out].
	out cr; tab; write: '^ self basicNew '.
	self writeInitialize: names on: out.
	class class compile: out conclusion classified: 'instance creation'
		
		! !

!ALEntityGenerator methodsFor: 'as yet unclassified' stamp: 'cwp 12/31/2012 10:55'!
generateGetter: aString index: anInteger
	| source |
	source := String new writing
		write: aString;
		cr;
		tab;
		write: 'vector ifNil: [vector := ALReadEntity key: key].';
		cr;
		tab;
		write: '^ vector at: ';
		print: anInteger;
		cr;
		conclusion.
	self compile: source classified: self accessorCategory! !

!ALEntityGenerator methodsFor: 'as yet unclassified' stamp: 'cwp 1/2/2013 22:53'!
generateInitialize: names
	| out |
	out := String new writing.
	self writeInitialize: names on: out.
	out tab; write: 'self initialize.'; cr.
	out tab; write: 'vector := ALVector new: '; 
		print: names size; put: $.; cr.
	names do: 
		[ :name | 
		out tab.
		out write: 'self '.
		self writeSet: name on: out.
		out put: $.; cr].
	out tab; write: 'key := vector add'.
	class compile: out conclusion classified: 'initialize-release'! !

!ALEntityGenerator methodsFor: 'as yet unclassified' stamp: 'cwp 12/31/2012 10:57'!
generateSetter: aString index: anInteger
	| source |
	source := String new writing
		write: aString;
		put: $:;
		write: ' anObject';
		cr;
		tab;
		write: 'vector ifNil: [vector := ALReadEntity key: key].';
		cr;
		tab;
		write: '^ vector at: ';
		print: anInteger;
		write: ' put: anObject';
		cr;
		conclusion.
	self compile: source classified: 'accessing'! !

!ALEntityGenerator methodsFor: 'as yet unclassified' stamp: 'cwp 12/31/2012 09:32'!
initializeWithClass: aClass
	self initialize.
	class := aClass! !

!ALEntityGenerator methodsFor: 'as yet unclassified' stamp: 'cwp 1/2/2013 19:22'!
writeInitialize: names on: out
	| first in |
	in := names reading.
	first := in get.
	out
		write: 'initializeWith';
		write: first capitalized;
		write: ': ';
		write: first.
	in
		do: [ :name | 
			out space.
			self writeSet: name on: out ].
	out cr! !

!ALEntityGenerator methodsFor: 'as yet unclassified' stamp: 'cwp 1/2/2013 19:11'!
writeSet: name on: out
	out write: name.
	out write: ': '.
	^ out write: name! !

Object subclass: #ALRepository
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AL-Storage-Repository'!

ALRepository subclass: #ALMemoryRepository
	instanceVariableNames: 'vectors'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AL-Storage-Repository'!

!ALMemoryRepository methodsFor: 'operations' stamp: 'cwp 1/3/2013 14:56'!
add: aKey vector: aVector
	aVector version: 1.
	vectors at: aKey copy put: aVector serialize.
	^ aKey! !

!ALMemoryRepository methodsFor: 'operations' stamp: 'cwp 11/24/2012 23:22'!
collect: keys
	^ keys collect: [:k | self read: k]! !

!ALMemoryRepository methodsFor: 'operations' stamp: 'cwp 12/31/2012 13:41'!
delete: key
	vectors removeKey: key ifAbsent: []! !

!ALMemoryRepository methodsFor: 'operations' stamp: 'cwp 12/31/2012 16:51'!
edit: key vector: aVector
	| old |
	old := self read: key.
	aVector version: old version + 1.
	vectors at: key put: aVector serialize.! !

!ALMemoryRepository methodsFor: 'initialize-release' stamp: 'cwp 12/31/2012 13:41'!
initialize
	vectors := Dictionary new! !

!ALMemoryRepository methodsFor: 'operations' stamp: 'cwp 1/3/2013 14:14'!
key
	| key byte |
	byte := 0 to: 255.
	[key := ByteArray new: 8.
	1 to: 8 do: [:i | key at: i put: byte atRandom].
	vectors includesKey: key] 
		whileTrue.
	^ key! !

!ALMemoryRepository methodsFor: 'operations' stamp: 'cwp 1/1/2013 23:00'!
read: aKey
	| archive vector |
	archive := vectors 
		at: aKey 
		ifAbsent: [ALStorageError signal: 'Entity doesn''t exist'].
	vector := archive materialize.
	vector primitiveChangeClassTo: ALWriteBarrierVector new.
	^ vector! !

!ALRepository methodsFor: 'operations' stamp: 'cwp 1/3/2013 14:57'!
add: aVector
	^ self add: self key vector: aVector! !

!ALRepository methodsFor: 'operations' stamp: 'cwp 1/3/2013 14:53'!
add: aKey vector: aVector
	self subclassResponsibility! !

!ALRepository methodsFor: 'operations' stamp: 'cwp 11/24/2012 23:24'!
collect: aCollection
	self subclassResponsibility! !

!ALRepository methodsFor: 'operations' stamp: 'cwp 11/24/2012 23:24'!
delete: aKey
	self subclassResponsibility! !

!ALRepository methodsFor: 'operations' stamp: 'cwp 12/31/2012 13:41'!
edit: aKey vector: aVector
	self subclassResponsibility! !

!ALRepository methodsFor: 'operations' stamp: 'cwp 1/3/2013 14:12'!
key
	self subclassResponsibility! !

!ALRepository methodsFor: 'operations' stamp: 'cwp 11/24/2012 23:25'!
read: aKey
	self subclassResponsibility! !

!ALRepository methodsFor: 'transacting' stamp: 'cwp 1/1/2013 23:42'!
readOnlyTransaction
	^ ALReadOnlyTransaction repository: self! !

!ALRepository methodsFor: 'transacting' stamp: 'cwp 1/1/2013 23:18'!
transaction
	^ ALReadWriteTransaction repository: self! !

Object subclass: #ALTransaction
	instanceVariableNames: 'repository reads'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AL-Storage-Core'!

ALTransaction subclass: #ALReadOnlyTransaction
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AL-Storage-Core'!

!ALReadOnlyTransaction methodsFor: 'transacting' stamp: 'cwp 1/2/2013 17:25'!
abort
	reads do: [ :v | v becomeForward: nil copyHash: false ]! !

!ALReadOnlyTransaction methodsFor: 'transacting' stamp: 'cwp 1/2/2013 17:29'!
commit
	
	reads keysAndValuesDo:
		[:key :vector |
		vector isDirty ifTrue: 
			[repository edit: key vector: vector].
		vector becomeForward: nil copyHash: false].
! !

!ALReadOnlyTransaction methodsFor: 'dispatch' stamp: 'cwp 1/1/2013 23:25'!
handleAddEntity: aNotification
	ALImmutableError signal! !

!ALReadOnlyTransaction methodsFor: 'dispatch' stamp: 'cwp 1/1/2013 23:25'!
handleDeleteEntity: aNotification
	ALImmutableError signal! !

!ALReadOnlyTransaction methodsFor: 'dispatch' stamp: 'cwp 1/1/2013 23:54'!
handleReadEntity: aNotification
	| key vector |
	key := aNotification key.
	reads at: key ifPresent: [:v | aNotification resume: v].
	vector := repository read: key.
	reads at: key put: vector.
	vector primitiveChangeClassTo: ALImmutableVector new.
	aNotification resume: vector
	! !

ALTransaction subclass: #ALReadWriteTransaction
	instanceVariableNames: 'adds deletes'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AL-Storage-Core'!

!ALReadWriteTransaction methodsFor: 'transacting' stamp: 'cwp 1/2/2013 17:39'!
abort
	reads do: [ :v | v becomeForward: nil copyHash: false].
	adds keys do: [:k | k becomeForward: nil copyHash: false].
	! !

!ALReadWriteTransaction methodsFor: 'accessing' stamp: 'cwp 1/1/2013 23:19'!
adds
	^ adds! !

!ALReadWriteTransaction methodsFor: 'transacting' stamp: 'cwp 1/11/2013 23:58'!
commit
	| opCount toNil |
	opCount := adds size + reads size + deletes size.
	toNil := (Array new: opCount) writing.
	adds keysAndValuesDo:
		[:key :vector |
		repository add: key vector: vector.
		toNil put: vector].
	
	reads keysAndValuesDo:
		[:key :vector |
		vector isDirty ifTrue: 
			[repository edit: key vector: vector].
		toNil put: vector].
	
	deletes do: 
		[:key | 
		repository delete: key.
		toNil put: key].
	
	toNil conclusion 
		elementsForwardIdentityTo: (Array new: opCount)
		copyHash: false.! !

!ALReadWriteTransaction methodsFor: 'accessing' stamp: 'cwp 1/1/2013 23:19'!
deletes
	^ deletes! !

!ALReadWriteTransaction methodsFor: 'dispatch' stamp: 'cwp 1/3/2013 14:14'!
handleAddEntity: aNotification
	| key |
	key := repository key.
	adds at: key put: aNotification vector.
	aNotification resume: key	! !

!ALReadWriteTransaction methodsFor: 'dispatch' stamp: 'cwp 1/1/2013 23:20'!
handleDeleteEntity: aNotification
	deletes add: aNotification key.
	aNotification resume! !

!ALReadWriteTransaction methodsFor: 'dispatch' stamp: 'cwp 1/1/2013 23:28'!
handleReadEntity: aNotification
	| key vector |
	key := aNotification key.
	
	(deletes includes: aNotification key) ifTrue:
		[ALStorageError signal: 'This entity has been deleted'].
	reads at: key ifPresent: [:v | aNotification resume: v].
	adds at: key ifPresent: [:v | aNotification resume: v].
	
	vector := repository read: key.
	reads at: key put: vector.
	aNotification resume: vector
	! !

!ALReadWriteTransaction methodsFor: 'initialization' stamp: 'cwp 1/1/2013 23:30'!
initializeWithRepository: aRepository
	super initializeWithRepository: aRepository.
	adds := Dictionary new.
	deletes := Set new.! !

!ALTransaction class methodsFor: 'as yet unclassified' stamp: 'cwp 12/31/2012 12:29'!
repository: aRepository	
	^ self basicNew initializeWithRepository: aRepository! !

!ALTransaction methodsFor: 'transacting' stamp: 'cwp 1/2/2013 17:25'!
abort
	self subclassResponsibility! !

!ALTransaction methodsFor: 'transacting' stamp: 'cwp 1/1/2013 23:22'!
abort: aBlock
	| result |
	result := self do: aBlock.
	self abort.
	^ result! !

!ALTransaction methodsFor: 'transacting' stamp: 'cwp 1/1/2013 23:27'!
commit
	self subclassResponsibility! !

!ALTransaction methodsFor: 'transacting' stamp: 'cwp 1/2/2013 17:26'!
commit: aBlock
	| result |
	result := self do: aBlock.
	self commit.
	^ result! !

!ALTransaction methodsFor: 'transacting' stamp: 'cwp 12/31/2012 12:37'!
do: aBlock
	^ aBlock 
		on: ALStorageNotification
		do: [:n | n handleWith: self]! !

!ALTransaction methodsFor: 'dispatch' stamp: 'cwp 1/1/2013 23:28'!
handleAddEntity: aNotification
	self subclassResponsibility! !

!ALTransaction methodsFor: 'dispatch' stamp: 'cwp 1/1/2013 23:28'!
handleDeleteEntity: aNotification
	self subclassResponsibility! !

!ALTransaction methodsFor: 'dispatch' stamp: 'cwp 1/1/2013 23:28'!
handleReadEntity: aNotification
	self subclassResponsibility! !

!ALTransaction methodsFor: 'initialization' stamp: 'cwp 1/1/2013 23:21'!
initializeWithRepository: aRepository
	self initialize.
	repository := aRepository.
	reads := Dictionary new.
! !

!ALTransaction methodsFor: 'accessing' stamp: 'cwp 1/1/2013 23:20'!
reads
	^ reads! !

Object variableSubclass: #ALVector
	instanceVariableNames: 'version'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AL-Storage-Core'!

ALVector variableSubclass: #ALImmutableVector
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AL-Storage-Core'!

!ALImmutableVector methodsFor: 'as yet unclassified' stamp: 'cwp 1/1/2013 23:01'!
at: anInteger put: anObject
	ALImmutableError signal! !

!ALImmutableVector methodsFor: 'as yet unclassified' stamp: 'cwp 1/1/2013 23:01'!
isDirty
	^ false! !

!ALVector methodsFor: 'as yet unclassified' stamp: 'cwp 12/31/2012 10:09'!
add
	^ ALAddEntity vector: self! !

!ALVector methodsFor: 'as yet unclassified' stamp: 'cwp 12/31/2012 17:06'!
isDirty
	^ true! !

!ALVector methodsFor: 'as yet unclassified' stamp: 'cwp 12/31/2012 12:42'!
version
	^ version! !

!ALVector methodsFor: 'as yet unclassified' stamp: 'cwp 12/31/2012 13:07'!
version: anObject
	version := anObject! !

ALVector variableSubclass: #ALWriteBarrierVector
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AL-Storage-Core'!

!ALWriteBarrierVector methodsFor: 'as yet unclassified' stamp: 'cwp 12/31/2012 17:08'!
at: anInteger put: anObject
	self primitiveChangeClassTo: ALVector new.
	self at: anInteger put: anObject.! !

!ALWriteBarrierVector methodsFor: 'as yet unclassified' stamp: 'cwp 12/31/2012 17:06'!
isDirty
	^ false! !
